@@ -190,6 +190,7 @@ class ADOHTTPClient:
         operations: Optional[List[PatchOperation]] = None,
         additional_headers: Optional[Dict[str, str]] = None,
         json_data: Optional[Any] = None,
+        stream: bool = False,
     ) -> requests.Response:
         """Issue a POST request with the correct credentials and headers.
 
@@ -200,6 +201,7 @@ class ADOHTTPClient:
         :param Optional[List[PatchOperation]] operations: The patch operations to send with the request
         :param Optional[Dict[str,str]] additional_headers: Any additional headers to add to the request
         :param Optional[Any] json_data: The JSON data to send with the request
+        :param bool stream: Set to True to stream the response back
 
         :returns: The raw response object from the API
         """
@@ -212,7 +214,9 @@ class ADOHTTPClient:
                 additional_headers["Content-Type"] = "application/json-patch+json"
 
         headers = self.construct_headers(additional_headers=additional_headers)
-        return requests.post(request_url, auth=self.credentials, headers=headers, json=json_data)
+        return requests.post(
+            request_url, auth=self.credentials, headers=headers, json=json_data, stream=stream
+        )
 
     @retry(
         retry=retry_if_exception(_is_connection_failure),
@@ -328,6 +332,22 @@ class ADOHTTPClient:
         response: requests.Response = session.send(prepped)
         return response
 
+    def validate_response(self, response: requests.models.Response) -> None:
+        """Checking a response for errors.
+
+        :param response: The response to check
+
+        :raises ADOHTTPException: Raised if the request returned a non-200 status code
+        :raises ADOException: Raise if the response was not JSON
+        """
+
+        self.log.debug("Validating response from ADO")
+
+        if response.status_code < 200 or response.status_code >= 300:
+            raise ADOHTTPException(
+                f"ADO returned a non-200 status code, configuration={self}", response,
+            )
+
     def decode_response(self, response: requests.models.Response) -> ADOResponse:
         """Decode the response from ADO, checking for errors.
 
@@ -339,12 +359,9 @@ class ADOHTTPClient:
         :raises ADOException: Raise if the response was not JSON
         """
 
-        self.log.debug("Fetching response from ADO")
+        self.validate_response(response)
 
-        if response.status_code < 200 or response.status_code >= 300:
-            raise ADOHTTPException(
-                f"ADO returned a non-200 status code, configuration={self}", response,
-            )
+        self.log.debug("Decoding response from ADO")
 
         try:
             content: ADOResponse = response.json()
