@@ -5,11 +5,10 @@
 
 """ADO work items API wrapper."""
 
-import datetime
 import enum
 import logging
 import os
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, cast, Dict, List, Optional
 
 from simple_ado.base_client import ADOBaseClient
 from simple_ado.context import ADOContext
@@ -17,6 +16,8 @@ from simple_ado.exceptions import ADOException, ADOHTTPException
 from simple_ado.http_client import ADOHTTPClient, ADOResponse
 from simple_ado.utilities import boolstr
 
+from simple_ado.models import PatchOperation, AddOperation
+
 
 class BatchRequest:
     """The base type for a batch request.
@@ -97,118 +98,6 @@ class WorkItemRelationType(enum.Enum):
     artifact_link = "ArtifactLink"
 
 
-class WorkItemField(enum.Enum):
-    """Defines the various fields available on a work item.
-
-    This does not include custom fields.
-    """
-
-    area_path = "/fields/System.AreaPath"
-    assigned_to = "/fields/System.AssignedTo"
-    changed_date = "/fields/System.ChangedDate"
-    closed_date = "/fields/Microsoft.VSTS.Common.ClosedDate"
-    created_by = "/fields/System.CreatedBy"
-    created_date = "/fields/System.CreatedDate"
-    description = "/fields/System.Description"
-    duplicate_id = "/fields/Office.Common.DuplicateID"
-    history = "/fields/System.History"
-    priority = "/fields/Microsoft.VSTS.Common.Priority"
-    resolved_date = "/fields/Microsoft.VSTS.Common.ResolvedDate"
-    resolved_reason = "/fields/Microsoft.VSTS.Common.ResolvedReason"
-    state = "/fields/System.State"
-    substatus = "/fields/Office.Common.SubStatus"
-    tags = "/fields/System.Tags"
-    title = "/fields/System.Title"
-
-    relation = "/relations/-"
-
-
-class WorkItemFieldOperationType(enum.Enum):
-    """Define the field operation types."""
-
-    add = "add"
-    copy = "copy"
-    move = "move"
-    remove = "remove"
-    replace = "replace"
-    test = "test"
-
-
-class WorkItemFieldOperation:
-    """Defines a base patch operation for sending via the ADO API.
-
-    :param operation: The operation type
-    :param path: The path to apply the operation to
-    :param value: The value (if any) to use
-    :param from_path: The original path (if required)
-    """
-
-    def __init__(
-        self,
-        operation: WorkItemFieldOperationType,
-        path: Union[WorkItemField, str],
-        value: Optional[str],
-        from_path: Optional[str] = None,
-    ) -> None:
-        self.operation = operation
-        self.path = path
-        self.value = value
-        self.from_path = from_path
-
-    def raw(self) -> Dict[str, Any]:
-        """Generate the raw representation that is sent to the API.
-
-        :returns: A dictionary with the raw API data for the request
-        """
-
-        if isinstance(self.value, datetime.datetime):
-            self.value = (
-                self.value.strftime("%Y-%m-%dT%H:%M:%S.")
-                + str(int(self.value.microsecond / 10000))
-                + "Z"
-            )
-
-        if isinstance(self.value, enum.Enum):
-            self.value = self.value.value
-
-        raw_dict = {"op": self.operation.value, "value": self.value, "from": self.from_path}
-
-        if isinstance(self.path, str):
-            raw_dict["path"] = self.path
-        else:
-            raw_dict["path"] = self.path.value
-
-        return raw_dict
-
-    def __str__(self) -> str:
-        """Generate and return the string representation of the object.
-
-        :return: A string representation of the object
-        """
-        return str(self.raw())
-
-
-class WorkItemFieldOperationAdd(WorkItemFieldOperation):
-    """Defines an add operation for sending via the ADO API.
-
-    :param field: The field to add
-    :param value: The value to set for the new field
-    """
-
-    def __init__(self, field: WorkItemField, value: Any) -> None:
-        super().__init__(WorkItemFieldOperationType.add, field, value)
-
-
-class WorkItemFieldOperationDelete(WorkItemFieldOperation):
-    """Defines a delete operation for sending via the ADO API.
-
-    :param field: The field to delete
-    """
-
-    def __init__(self, field: WorkItemField) -> None:
-        super().__init__(WorkItemFieldOperationType.remove, field, None)
-
-
 class ADOWorkItemsClient(ADOBaseClient):
     """Wrapper class around the ADO work items APIs.
 
@@ -231,9 +120,7 @@ class ADOWorkItemsClient(ADOBaseClient):
         """
 
         self.log.debug(f"Getting work item: {identifier}")
-        request_url = (
-            f"{self.http_client.base_url()}/wit/workitems/{identifier}?api-version=4.1&$expand=all"
-        )
+        request_url = f"{self.http_client.api_endpoint()}/wit/workitems/{identifier}?api-version=4.1&$expand=all"
         response = self.http_client.get(request_url)
         return self.http_client.decode_response(response)
 
@@ -243,15 +130,15 @@ class ADOWorkItemsClient(ADOBaseClient):
         :returns: The ADO response with the data in it
         """
         self.log.debug("Getting work item types")
-        request_url = f"{self.http_client.base_url()}/wit/workitemtypes?api-version=4.1"
+        request_url = f"{self.http_client.api_endpoint()}/wit/workitemtypes?api-version=4.1"
         response = self.http_client.get(request_url)
         return self.http_client.decode_response(response)
 
     def add_property(
         self,
         identifier: str,
-        field: WorkItemField,
-        value: Any,
+        field: str,
+        value: str,
         *,
         bypass_rules: bool = False,
         supress_notifications: bool = False,
@@ -270,18 +157,18 @@ class ADOWorkItemsClient(ADOBaseClient):
         :returns: The ADO response with the data in it
         """
 
-        self.log.debug(f"Add field '{field.value}' to ticket {identifier}")
+        self.log.debug(f"Add field '{field}' to ticket {identifier}")
 
-        operation = WorkItemFieldOperationAdd(field, value)
+        operation = AddOperation(field, value)
 
-        request_url = f"{self.http_client.base_url()}/wit/workitems/{identifier}"
+        request_url = f"{self.http_client.api_endpoint()}/wit/workitems/{identifier}"
         request_url += f"?bypassRules={boolstr(bypass_rules)}"
         request_url += f"&suppressNotifications={boolstr(supress_notifications)}"
         request_url += f"&api-version=4.1"
 
         response = self.http_client.patch(
             request_url,
-            [operation.raw()],
+            operations=[operation],
             additional_headers={"Content-Type": "application/json-patch+json"},
         )
 
@@ -321,7 +208,7 @@ class ADOWorkItemsClient(ADOBaseClient):
 
         # Upload the file
         request_url = (
-            f"{self.http_client.base_url()}/wit/attachments?fileName={filename}&api-version=1.0"
+            f"{self.http_client.api_endpoint()}/wit/attachments?fileName={filename}&api-version=1.0"
         )
 
         response = self.http_client.post_file(request_url, path_to_attachment)
@@ -334,19 +221,18 @@ class ADOWorkItemsClient(ADOBaseClient):
             raise ADOException(f"Failed to get url from response: {response_data}")
 
         # Attach it to the ticket
-        operation = WorkItemFieldOperationAdd(
-            WorkItemField.relation,
-            {"rel": "AttachedFile", "url": url, "attributes": {"comment": ""}},
+        operation = AddOperation(
+            "/relations/-", {"rel": "AttachedFile", "url": url, "attributes": {"comment": ""}},
         )
 
-        request_url = f"{self.http_client.base_url()}/wit/workitems/{identifier}"
+        request_url = f"{self.http_client.api_endpoint()}/wit/workitems/{identifier}"
         request_url += f"?bypassRules={boolstr(bypass_rules)}"
         request_url += f"&suppressNotifications={boolstr(supress_notifications)}"
         request_url += f"&api-version=4.1"
 
         response = self.http_client.patch(
             request_url,
-            [operation.raw()],
+            operations=[operation],
             additional_headers={"Content-Type": "application/json-patch+json"},
         )
 
@@ -378,19 +264,19 @@ class ADOWorkItemsClient(ADOBaseClient):
 
         self.log.debug(f"Adding link {parent_identifier} -> {child_url} ({relation_type})")
 
-        operation = WorkItemFieldOperationAdd(
-            WorkItemField.relation,
+        operation = AddOperation(
+            "/relations/-",
             {"rel": relation_type.value, "url": child_url, "attributes": {"comment": ""}},
         )
 
-        request_url = f"{self.http_client.base_url()}/wit/workitems/{parent_identifier}"
+        request_url = f"{self.http_client.api_endpoint()}/wit/workitems/{parent_identifier}"
         request_url += f"?bypassRules={boolstr(bypass_rules)}"
         request_url += f"&suppressNotifications={boolstr(supress_notifications)}"
         request_url += f"&api-version=4.1"
 
         response = self.http_client.patch(
             request_url,
-            [operation.raw()],
+            operations=[operation],
             additional_headers={"Content-Type": "application/json-patch+json"},
         )
 
@@ -420,7 +306,7 @@ class ADOWorkItemsClient(ADOBaseClient):
         :returns: The ADO response with the data in it
         """
         child_url = (
-            f"{self.http_client.base_url(is_project=False)}/wit/workitems/{child_identifier}"
+            f"{self.http_client.api_endpoint(is_project=False)}/wit/workitems/{child_identifier}"
         )
         return self._add_link(
             parent_identifier=parent_identifier,
@@ -461,7 +347,7 @@ class ADOWorkItemsClient(ADOBaseClient):
     def create(
         self,
         item_type: str,
-        operations: List[WorkItemFieldOperationAdd],
+        operations: List[AddOperation],
         *,
         bypass_rules: bool = False,
         supress_notifications: bool = False,
@@ -482,14 +368,14 @@ class ADOWorkItemsClient(ADOBaseClient):
 
         self.log.debug(f"Creating a new {item_type}")
 
-        request_url = f"{self.http_client.base_url()}/wit/workitems/${item_type}"
+        request_url = f"{self.http_client.api_endpoint()}/wit/workitems/${item_type}"
         request_url += f"?bypassRules={boolstr(bypass_rules)}"
         request_url += f"&suppressNotifications={boolstr(supress_notifications)}"
         request_url += f"&api-version=4.1"
 
         response = self.http_client.post(
             request_url,
-            json_data=[operation.raw() for operation in operations],
+            operations=cast(List[PatchOperation], operations),
             additional_headers={"Content-Type": "application/json-patch+json"},
         )
 
@@ -498,7 +384,7 @@ class ADOWorkItemsClient(ADOBaseClient):
     def update(
         self,
         identifier: str,
-        operations: List[WorkItemFieldOperation],
+        operations: List[PatchOperation],
         *,
         bypass_rules: bool = False,
         supress_notifications: bool = False,
@@ -518,14 +404,14 @@ class ADOWorkItemsClient(ADOBaseClient):
 
         self.log.debug(f"Updating {identifier}")
 
-        request_url = f"{self.http_client.base_url()}/wit/workitems/{identifier}"
+        request_url = f"{self.http_client.api_endpoint()}/wit/workitems/{identifier}"
         request_url += f"?bypassRules={boolstr(bypass_rules)}"
         request_url += f"&suppressNotifications={boolstr(supress_notifications)}"
         request_url += f"&api-version=4.1"
 
         response = self.http_client.patch(
             request_url,
-            [operation.raw() for operation in operations],
+            operations=operations,
             additional_headers={"Content-Type": "application/json-patch+json"},
         )
 
@@ -541,9 +427,9 @@ class ADOWorkItemsClient(ADOBaseClient):
 
         self.log.debug(f"Executing query: {query_string}")
 
-        request_url = f"{self.http_client.base_url()}/wit/wiql?api-version=4.1"
+        request_url = f"{self.http_client.api_endpoint()}/wit/wiql?api-version=4.1"
 
-        response = self.http_client.post(request_url, {"query": query_string})
+        response = self.http_client.post(request_url, json_data={"query": query_string})
 
         return self.http_client.decode_response(response)
 
@@ -566,7 +452,7 @@ class ADOWorkItemsClient(ADOBaseClient):
 
         self.log.debug(f"Deleting {identifier}")
 
-        request_url = f"{self.http_client.base_url()}/wit/workitems/{identifier}"
+        request_url = f"{self.http_client.api_endpoint()}/wit/workitems/{identifier}"
         request_url += f"?suppressNotifications={boolstr(supress_notifications)}"
         request_url += f"&destroy={boolstr(permanent)}"
         request_url += f"&api-version=4.1"
@@ -599,8 +485,8 @@ class ADOWorkItemsClient(ADOBaseClient):
         for operation in operations:
             full_body.append(operation.body())
 
-        request_url = f"{self.http_client.base_url(is_project=False)}/wit/$batch"
+        request_url = f"{self.http_client.api_endpoint(is_project=False)}/wit/$batch"
 
-        response = self.http_client.post(request_url, full_body)
+        response = self.http_client.post(request_url, json_data=full_body)
 
         return self.http_client.decode_response(response)
